
"""GDP: The global drifter program

Reference:
https://www.aoml.noaa.gov/phod/gdp/

"""
import os
import glob
import pathlib
import ftplib
from urllib.parse import urlsplit



import numpy as np
import pandas as pd

import requests
import click

DATADIR = os.path.expanduser("~/.oceandata/")
pathlib.Path(DATADIR).mkdir(parents=True, exist_ok=True)


def read_dat(filename, sst=False, vel=False, var=False):
    df = pd.read_csv(filename, sep=" ", skipinitialspace=True,
                     compression='gzip', na_values=999.999,
                     names=["id", "month", "day", "year", "lat", "lon",
                            "sst", "vel_east", "vel_north", "speed",
                            "var_lat", "var_lon", "var_temp"])
    df["hour"] = ((df.day - df["day"].astype(int)) * 24).astype(np.int32)
    df.set_index(pd.to_datetime(df[["year","month","day","hour"]]),inplace=True)
    del df["year"], df["month"], df["day"], df["hour"]
    df.loc[df["lon"]>180, "lon"] = df.loc[df["lon"]>180, "lon"] - 360
    if not sst:
        del df["sst"]
    if not vel:
        del df["vel_east"], df["vel_north"], df["speed"]
    if not var:
        del df["var_lat"], df["var_lon"], df["var_temp"]
    return df

def load(v1=None, sst=False, vel=False, var=False):
    """Load gzipped dat file to a pandas dataframe"""
    if v1 is None:
        dflist = []
        for v1 in [1, 5001, 10001, 15001]:
            dflist.append(load(v1, sst=sst, vel=vel, var=var))
        return pd.concat(dflist)
    if   v1 == 1:
        v2 = 5000
    elif v1 == 5001:
        v2 = 10000
    elif v1 == 10001:
        v2 = 15000
    elif v1 == 15001:
        v2 = "oct18"
    filename = os.path.join(DATADIR, f"buoydata_{v1}_{v2}.dat.gz")
    h5filename = filename[:-7] + ".h5"
    if os.path.isfile(h5filename):
        return pd.read_hdf(h5filename)
    if not os.path.isfile(filename):
        print("Downloading file")
        download(v1=v1, v2=v2)
    df = read_dat(filename, sst=sst, vel=vel, var=var)
    df.to_hdf(h5filename, "df")
    return df

def vprint(text):
    pass
    #print(text)

def download(url="ftp://ftp.aoml.noaa.gov/phod/pub/buoydata/",
             v1="15001", v2="oct18"):
    """Download tsv file from NOOA's website using ftp"""
    lfn = filename = f"buoydata_{v1}_{v2}.dat.gz"
    spliturl = urlsplit(url)
    try:
        ftp = ftplib.FTP(spliturl.netloc) 
        vprint(ftp.login("anonymous", "oceandata@bror.us"))
        ftpdir = spliturl.path
        vprint("Change dir to '%s'" % ftpdir)
        vprint(ftp.cwd(ftpdir))
    except ftplib.error_perm as err:
        print (spliturl.netloc)
        print (os.path.split(spliturl.path)[0])
        raise IOError(err)

    local_filename = os.path.join(DATADIR, filename)
    if not filename in ftp.nlst():
        print(ftp.nlst())
        raise ftplib.Error("'%s' is not the ftp server" % lfn)
    with open(local_filename, 'wb') as lfh:
        ftp.voidcmd('TYPE I')
        length = ftp.size(lfn)
        short_lfn = lfn if len(lfn)<18 else lfn[:4] + "..." + lfn[-13:]
        with click.progressbar(length=length, label=short_lfn) as bar:
            def file_write(data):
                lfh.write(data) 
                bar.update(len(data))
            try:
                ftp.retrbinary("RETR %s" % lfn, file_write)
            except ftplib.error_perm as err:
                os.unlink(local_filename)
                raise IOError(err)
        ftp.quit()

def unzip(filename="SOCATv6.tsv.zip"):
    local_filename = os.path.join(DATADIR, filename)
    zip_ref = zipfile.ZipFile(local_filename, 'r')
    zip_ref.extractall(DATADIR)
    zip_ref.close()




import os
"""SOCAT: photosynthesis-irradiance parameters for marine phytoplankton


Data DOI:
DOI:10.1594/PANGAEA.874087

References:
DOI:10.5194/essd-10-251-2018

"""
import glob
import pathlib
import zipfile

import numpy as np
import pandas as pd

import requests
import click

DATADIR = os.path.expanduser("~/.oceandata/")
pathlib.Path(DATADIR).mkdir(parents=True, exist_ok=True)

def load(filename="SOCATv6.tsv.zip"):
    """Load tsv file and fix some columns"""
    df = pd.read_csv(os.path.join(DATADIR, FILENAME), sep="\t", skiprows=57)
    df["lat"]    = df["Latitude"]
    df["lon"]    = df["Longitude"]
    df["region"] = df["BG province"]
    df["depth"]  = df["Depth water [m]"]
    df["chl"]    = df["Chl a [µg/l]"]
    df["alpha"]  = df["alpha [(mg C/mg Chl a/h)/(µE/m**2/s)]"] 
    df["PBmax"]  = df["PBmax [mg C/mg Chl a/h]"]
    df["Ek"]     = df["Ek [µmol/m**2/s]"]
    del df["Latitude"], df["Longitude"], df["BG province"]
    del df["Depth water [m]"], df["Chl a [µg/l]"]
    del df["alpha [(mg C/mg Chl a/h)/(µE/m**2/s)]"]
    del df["PBmax [mg C/mg Chl a/h]"], df["Ek [µmol/m**2/s]"]
    df.set_index(pd.DatetimeIndex(df["Date/Time"]), inplace=True)
    del df["Date/Time"]
    return df


def download(url="https://www.socat.info/socat_files/", version="v6",
             filename="SOCATv6.tsv.zip"):
    """Download tsv file from Pangaea server"""
    local_filename = os.path.join(DATADIR, filename)
    try:
        r = requests.get(url=f"{url}/{version}/{filename}",
                         stream=True, timeout=20)
    except requests.ReadTimeout:
        warnings.warn("Connection to server timed out.")
        return False
    if r.ok:
        total_size = int(r.headers.get('Content-Length'))
        if local_filename is None:
            return r.text
        else:
            with open(local_filename, 'wb') as f:
                with click.progressbar(length=total_size,
                                       label='Downloading files') as bar:
                    for chunk in r.iter_content(chunk_size=1024): 
                        if chunk:
                            f.write(chunk)
                            f.flush()
                            bar.update(len(chunk))
            return None
    else:
        raise IOError("Could not download file from server")

def unzip(filename="SOCATv6.tsv.zip"):
    local_filename = os.path.join(DATADIR, filename)
    zip_ref = zipfile.ZipFile(local_filename, 'r')
    zip_ref.extractall(DATADIR)
    zip_ref.close()



# -*- coding: utf-8 -*-
from distutils.core import setup

package_dir = \
{'': 'src'}

packages = \
['oceandata']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0',
 'numpy>=1.16,<2.0',
 'pandas>=0.24.1,<0.25.0',
 'requests>=2.21,<3.0']

setup_kwargs = {
    'name': 'oceandata',
    'version': '0.2.2',
    'description': '',
    'long_description': None,
    'author': 'Bror Jonsson',
    'author_email': 'brorfred@gmail.com',
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)




import os
"""MAPPS: photosynthesis-irradiance parameters for marine phytoplankton

The MAPPS global database of photosynthesis-irradiance (P-E) parameters consists of over 5000 P-E experiments that provides information on the spatio-temporal variability in the two P-E parameters (the assimilation number, and the initial slope) that are fundamental inputs for models of marine primary production that use chlorophyll as the state variable. The experiments were carried out by an international group of research scientists to examine the basin-scale variability in the photophysiological response of marine phytoplankton over a range of oceanic regimes (from the oligotrophic gyres to productive shelf systems) and covers several decades. These data can be used to improve the assignment of P-E parameters in the estimation of marine primary production using satellite dat

Data DOI:
DOI:10.1594/PANGAEA.874087

References:
DOI:10.5194/essd-10-251-2018

"""
import glob
import pathlib

import numpy as np
import pandas as pd

import requests

DATADIR = os.path.expanduser("~/.oceandata/")
FILENAME = "Bouman_2017.tab.tsv"
pathlib.Path(DATADIR).mkdir(parents=True, exist_ok=True)

def load(filename=FILENAME):
    """Load tsv file and fix some columns"""
    df = pd.read_csv(os.path.join(DATADIR, FILENAME), sep="\t", skiprows=57)
    df["lat"]    = df["Latitude"]
    df["lon"]    = df["Longitude"]
    df["region"] = df["BG province"]
    df["depth"]  = df["Depth water [m]"]
    df["chl"]    = df["Chl a [µg/l]"]
    df["alpha"]  = df["alpha [(mg C/mg Chl a/h)/(µE/m**2/s)]"] 
    df["PBmax"]  = df["PBmax [mg C/mg Chl a/h]"]
    df["Ek"]     = df["Ek [µmol/m**2/s]"]
    del df["Latitude"], df["Longitude"], df["BG province"]
    del df["Depth water [m]"], df["Chl a [µg/l]"]
    del df["alpha [(mg C/mg Chl a/h)/(µE/m**2/s)]"]
    del df["PBmax [mg C/mg Chl a/h]"], df["Ek [µmol/m**2/s]"]
    df.set_index(pd.DatetimeIndex(df["Date/Time"]), inplace=True)
    del df["Date/Time"]
    return df


def download(url="https://doi.pangaea.de/10.1594/PANGAEA.874087",
            params={"format":"textfile"}, filename=None):
    """Download tsv file from Pangaea server"""
    filename = FILENAME if filename is None else filename
    local_filename = os.path.join(DATADIR, filename)
    try:
        r = requests.get(url, params=params, stream=True, timeout=2)
    except requests.ReadTimeout:
        warnings.warn("Connection to server timed out.")
        return False
    if r.ok:
        if local_filename is None:
            return r.text
        else:
            with open(local_filename, 'wb') as f:
                for chunk in r.iter_content(chunk_size=1024): 
                    if chunk:
                        f.write(chunk)
                        f.flush()
            return None
    else:
        raise IOError("Could not download file from server")


def download_pml(url="https://github.com/brorfred/oceandata/raw/master/data/",
                filename="GLOBAL_PE_W_LOV_2019.csv", params={}):
    """Download local PML version of MAPPS"""
    download(url=f"{url}/{filename}", filename=filename, params=params)


def load_pml(filename="GLOBAL_PE_W_LOV_2019.csv"):
    fn = os.path.join(DATADIR, filename)
    df = pd.read_csv(fn, parse_dates=[[4,5,6]], index_col="YEAR_MONTH_DAY")
    df = df.rename(columns={"LAT":"lat", "LON":"lon", "DEPTH":"depth",
                            "TEMP":"temp", "TCHL":"chl", "ALPHA":"alpha",
                            'NITRATE':"NO3",'SILICATE':"Si4",'PHOSPHATE':"PO4",
                            "PMB":"PBmax", "EK":"Ek","PROVNUM":"region"})
    return df
